import logging
from pathlib import Path, PurePosixPath, PureWindowsPath, WindowsPath


class Store:
    """Object to handle different paths to a store amongst computers"""

    def __init__(self, local_path: str = None, win_path: str = None, unix_path: str = None):
        """Initialize Store object

        Args:
            local_path (str, optional): Path to the store on the local computer. If not provided, it is supposed to be
            the same as either `win_runner_path` or `unix_runner_path` based on the OS.
            Defaults to None.
            win_path (str, optional): Path to the store on the windows runner computers. Defaults to None.
            unix_path (str, optional): Path to the store on the unix runner computers. Defaults to None.
        """
        if not local_path:
            if isinstance(Path("a"), WindowsPath):
                local_path = win_path
            else:
                local_path = unix_path

        self._local_path = Path(local_path).resolve()  # resolve mount point in case of windows mount
        self._win_path = PureWindowsPath(win_path) if win_path is not None else None
        self._unix_path = PurePosixPath(unix_path) if unix_path is not None else None

    def to_unix(self, dir: str | Path) -> PurePosixPath | None:
        if dir == "":
            return None
        if self._unix_path is None:
            raise ValueError("Trying to get Unix path from store which has unix_path not set")

        dir_path = Path(dir).resolve()
        if dir_path.is_relative_to(self._local_path):
            relative_path = dir_path.relative_to(self._local_path)
            out_path = self._unix_path / PurePosixPath(relative_path)
        else:
            logging.warning(f"Path {dir_path} has not been detected as relative to store path ({self._local_path})")
            out_path = PurePosixPath(dir_path)

        return out_path

    def to_win(self, dir: str | Path) -> PureWindowsPath | None:
        if dir == "":
            return None
        if self._win_path is None:
            raise ValueError("Trying to get Windows path from store which has win_path not set")

        dir_path = Path(dir).resolve()
        if dir_path.is_relative_to(self._local_path):
            relative_path = dir_path.relative_to(self._local_path)
            out_path = self._win_path / PureWindowsPath(relative_path)
        else:
            logging.warning(f"Path {dir_path} has not been detected as relative to store path ({self._local_path})")
            out_path = PureWindowsPath(dir_path)

        return out_path

    def win_to_local(self, dir: str | PureWindowsPath) -> Path:
        dir_path = PureWindowsPath(dir)
        if dir_path.is_relative_to(self._win_path):
            relative_path = dir_path.relative_to(self._win_path)
            out_path = self._local_path / Path(relative_path)
        else:
            logging.warning(
                f"Path {dir_path} has not been detected as relative to windows store path ({self._win_path})"
            )
            out_path = Path(dir_path)
        return out_path

    def unix_to_local(self, dir: str | PurePosixPath) -> Path:
        dir_path = PurePosixPath(dir)
        if dir_path.is_relative_to(self._unix_path):
            relative_path = dir_path.relative_to(self._unix_path)
            out_path = self._local_path / Path(relative_path)
        else:
            logging.warning(
                f"Path {dir_path} has not been detected as relative to unix store path ({self._unix_path})"
            )
            out_path = Path(dir_path)
        return out_path



import time

import requests
from client import worker
from pytest import fail


def is_all_job_done_for_project_starting_with(url: str, project_name: str, pytrace=True):
    """This method fails if any job is found failed"""
    response = worker.send_request(url + "jobs", "GET")
    all_jobs_done = True
    found_at_least_one_job = False

    if response and response.json():
        for job in response.json():
            if job["project_name"].startswith(project_name):
                found_at_least_one_job = True
                status = job["job_status"]
                if status == "failed":
                    fail("job named '" + job["job_name"] + "' has status FAILED", pytrace)
                if status != "done":
                    all_jobs_done = False

    if not found_at_least_one_job:
        fail("There is not any project starting with: " + project_name)
    return all_jobs_done


def pretty_time_delta(seconds):
    sign_string = "-" if seconds < 0 else ""
    seconds = abs(int(seconds))
    days, seconds = divmod(seconds, 86400)
    hours, seconds = divmod(seconds, 3600)
    minutes, seconds = divmod(seconds, 60)
    if days > 0:
        return "%s%dd%dh%dm%ds" % (sign_string, days, hours, minutes, seconds)
    elif hours > 0:
        return "%s%dh%dm%ds" % (sign_string, hours, minutes, seconds)
    elif minutes > 0:
        return "%s%dm%ds" % (sign_string, minutes, seconds)
    else:
        return "%s%ds" % (sign_string, seconds)


def wait_running_job(url_api, project_name, delay_second=1, delay_log_second=10):
    all_jobs_done = False
    duration_second = 0

    while not all_jobs_done:
        time.sleep(delay_second)
        duration_second = duration_second + delay_second
        if duration_second % delay_log_second == 0:
            print("wait for job to finish since " + pretty_time_delta(duration_second))

        all_jobs_done = is_all_job_done_for_project_starting_with(url_api, project_name)


def start_threads(url_api, num_thread, pattern=""):
    hosts_list = []
    response = worker.send_request(url_api + "nodes", "GET")
    if response and response.json():
        for host in response.json():
            hostname = host["host"]
            if pattern == "" or pattern in hostname:
                hosts_list.append(hostname)

    json_host = {"hosts": hosts_list}
    params = {"value": str(num_thread)}
    response = requests.post(url_api + "node/setNbActive", params=params, json=json_host)
    assert response.status_code == 200


def assert_all_jobs_are_done_in_project(project_name: str, pytrace=True):
    """Verify that all job in this project are Done. Verify this for all project starting with this name"""
    response = worker.send_request(worker.GPAO_API_URL + "jobs", "GET")
    any_job_for_this_project_name = False

    if response and response.json():
        for job in response.json():
            if job["project_name"] == project_name:
                any_job_for_this_project_name = True
                status = job["job_status"]
                if status != "done":
                    fail(
                        "job named '" + job["job_name"] + "' has status : " + status,
                        pytrace,
                    )

    if not any_job_for_this_project_name:
        fail(f"There is not any job for this project name {project_name}")


def delete_project(project_name: str, starts_with: bool = False):
    """Delete GPAO projects that have this name, or starting with this name.
    Args:
        project_name (str): Name of the project to delete
        starts_with (bool, optional): If true, delete all GPAO project starting with 'project_name' param.
        Defaults to False.
    """
    response = worker.send_request(worker.GPAO_API_URL + "projects", "GET")
    id_list = []
    if response and response.json():
        for proj in response.json():
            name = proj["project_name"]
            if (starts_with and name.startswith(project_name)) or (name == project_name):
                id = proj["project_id"]
                id_list.append(id)

    if len(id_list) > 0:
        response = requests.delete(worker.GPAO_API_URL + "projects/delete", json={"ids": id_list})



import PySimpleGUI as sg

from gpao_utils import interface_utils as iu


def frame_store(stores: list):
    layout_store = []
    for i, store in enumerate(stores):
        layout_store.append([sg.Text(store.upper())])
        layout_store.append(
            [
                sg.Text(f" {i} - Chemin local vers le store", size=(iu.size_text, 1)),
                sg.InputText(
                    sg.user_settings_get_entry(f"-MAP-STORE{i}-LOCAL", ""),
                    key=f"-MAP-STORE{i}-LOCAL",
                    size=(iu.size_text_big, 1),
                ),
            ]
        )
        layout_store.append(
            [
                sg.Text(f" {i} - store Windows (sur le noeud de calcul)", size=(iu.size_text, 1)),
                sg.InputText(
                    sg.user_settings_get_entry(f"-MAP-STORE{i}-WIN", f"//store.ign.fr/{store}"),
                    key=f"-MAP-STORE{i}-WIN",
                    size=(iu.size_text_big, 1),
                ),
            ]
        )
        layout_store.append(
            [
                sg.Text(f" {i} - store Linux (sur le noeud de calcul)", size=(iu.size_text, 1)),
                sg.InputText(
                    sg.user_settings_get_entry(f"-MAP-STORE{i}-UNIX", f"/var/data/{store}"),
                    key=f"-MAP-STORE{i}-UNIX",
                    size=(iu.size_text_big, 1),
                ),
            ]
        )
    return sg.Frame("", layout_store)



import sys
from pathlib import Path, PosixPath, PurePosixPath, PureWindowsPath, WindowsPath

import pytest

from gpao_utils.store import Store

if isinstance(Path("a"), WindowsPath):
    local_path = Path("L:/")
else:
    local_path = Path("/mnt/store1")


def test_to_unix_ok():
    store = Store(local_path, "//store.example.fr/store1/", "/mnt/store1/")
    path_to_test = local_path / "toto.txt"
    assert store.to_unix(path_to_test) == PurePosixPath("/mnt/store1/toto.txt")


def test_to_unix_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # resolve path before sending to store to make sure it works on the current platform
    path_to_test = Path("./toto.txt").resolve()
    assert store.to_unix(path_to_test) == PurePosixPath(path_to_test)


def test_to_unix_nok():
    store = Store(local_path, "//store.example.fr/store1/", None)
    path_to_test = local_path / "toto.txt"
    with pytest.raises(ValueError):
        store.to_unix(path_to_test)


def test_to_win_ok():
    store = Store(local_path, "//store.example.fr/store1/", "/mnt/store1/")
    path_to_test = local_path / "toto.txt"
    assert store.to_win(path_to_test) == PureWindowsPath("//store.example.fr/store1/toto.txt")


def test_to_win_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # resolve path before sending to store to make sure it works on the current platform
    path_to_test = Path("./toto.txt").resolve()
    assert store.to_win(path_to_test) == PureWindowsPath(path_to_test)


def test_to_win_nok():
    store = Store(local_path, None, "/mnt/store1/")
    path_to_test = local_path / "toto.txt"
    with pytest.raises(ValueError):
        assert store.to_win(path_to_test)


def test_to_win_case():
    store = Store(local_path, "//store.example.fr/store1/", "/mnt/store1/")
    path_to_test = local_path / "TOTO.txt"
    assert store.to_win(path_to_test) == PureWindowsPath("//store.example.fr/store1/toto.txt")


def test_to_win_separator():
    if isinstance(Path("a"), WindowsPath):
        local_path = Path("\\\\store.example.fr\\store1")
    else:
        local_path = Path("/mnt/store1")
    store = Store(local_path, "\\\\store.example.fr\\store2", "/mnt/store1/")
    path_to_test = local_path / "toto.txt"
    assert store.to_win(path_to_test) == PureWindowsPath("\\\\store.example.fr\\store2\\toto.txt")


def test_no_local_path():
    win_path = PureWindowsPath("//store1.example.fr/win_path")
    unix_path = PurePosixPath("/mnt/store1/unix_store")
    store = Store(win_path=win_path, unix_path=unix_path)
    if isinstance(Path("a"), WindowsPath):
        assert store._local_path == win_path
    else:
        assert store._local_path == unix_path


def test_to_unix_empty_string():
    store = Store(win_path="//store.ign.fr/store1", unix_path="/mnt/store1")
    assert store.to_unix("") is None


def test_to_win_empty_string():
    store = Store(win_path="//store.ign.fr/store1", unix_path="/mnt/store1")
    assert store.to_win("") is None


###################
# test win_to_local
###################


@pytest.mark.skipif(sys.platform.startswith("win"), reason="requires unix platform")
def test_win_to_local_on_unix_ok():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    assert store.win_to_local("//store.example.fr/store1/my/path") == PosixPath("/mnt/store1/my/path")


@pytest.mark.skipif(not sys.platform.startswith("win"), reason="requires windows platform")
def test_win_to_local_on_windows_ok():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    assert store.win_to_local("//store.example.fr/store1/my/path") == WindowsPath("L:/my/path")


@pytest.mark.skipif(sys.platform.startswith("win"), reason="requires unix platform")
def test_win_to_local_on_unix_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # Does not change the path but store it in a PosixPath object
    assert store.win_to_local("//my/other/path") == PosixPath("//my/other/path")


@pytest.mark.skipif(not sys.platform.startswith("win"), reason="requires windows platform")
def test_win_to_local_on_windows_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # Does not change the path but store it in a WindowsPath object
    assert store.win_to_local("//my/other/path") == WindowsPath("//my/other/path")


####################
# test unix_to_local
####################


@pytest.mark.skipif(sys.platform.startswith("win"), reason="requires unix platform")
def test_unix_to_local_on_unix_ok():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    assert store.unix_to_local("/mnt/store1/my/path") == PosixPath("/mnt/store1/my/path")


@pytest.mark.skipif(not sys.platform.startswith("win"), reason="requires windows platform")
def test_unix_to_local_on_windows_ok():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    assert store.unix_to_local("/mnt/store1/my/path") == WindowsPath("L:/my/path")


@pytest.mark.skipif(sys.platform.startswith("win"), reason="requires unix platform")
def test_unix_to_local_on_unix_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # Does not change the path but store it in a PosixPath object
    assert store.unix_to_local("/my/other/path") == PosixPath("/my/other/path")


@pytest.mark.skipif(not sys.platform.startswith("win"), reason="requires windows platform")
def test_unix_to_local_on_windows_not_relative():
    store = Store(local_path, "//store.example.fr/store1", "/mnt/store1/")
    # Does not change the path but store it in a WindowsPath object
    assert store.unix_to_local("/my/other/path") == WindowsPath("/my/other/path")
